@@ -1,15 +1,16 @@
 use super::ast::*;
-use super::common::{ImmutableString, Range};
+use super::common::Range;
 use super::errors::*;
 use super::scanner::Scanner;
 use super::tokens::{Token, TokenAndRange};
+use std::borrow::Cow;
 use std::collections::HashMap;
 use std::rc::Rc;
 
 /// Map where the comments are stored in collections where
 /// the key is the previous token end or start of file or
 /// next token start or end of the file.
-pub type CommentMap = HashMap<usize, Rc<Vec<Comment>>>;
+pub type CommentMap<'a> = HashMap<usize, Rc<Vec<Comment<'a>>>>;
 
 /// Options for parsing.
 #[derive(Default)]
@@ -21,32 +22,32 @@ pub struct ParseOptions {
 }
 
 /// Result of parsing the text.
-pub struct ParseResult {
+pub struct ParseResult<'a> {
     /// Collection of comments in the text.
     ///
     /// Provide `comments: true` to the `ParseOptions` for this to have a value.
     ///
     /// Remarks: The key is the start and end position of the tokens.
-    pub comments: Option<CommentMap>,
+    pub comments: Option<CommentMap<'a>>,
     /// The JSON value the text contained.
-    pub value: Option<Value>,
+    pub value: Option<Value<'a>>,
     /// Collection of tokens (excluding any comments).
     ///
     /// Provide `tokens: true` to the `ParseOptions` for this to have a value.
-    pub tokens: Option<Vec<TokenAndRange>>,
+    pub tokens: Option<Vec<TokenAndRange<'a>>>,
 }
 
-struct Context {
-    scanner: Scanner,
-    comments: Option<CommentMap>,
-    current_comments: Option<Vec<Comment>>,
+struct Context<'a> {
+    scanner: Scanner<'a>,
+    comments: Option<CommentMap<'a>>,
+    current_comments: Option<Vec<Comment<'a>>>,
     last_token_end: usize,
     range_stack: Vec<Range>,
-    tokens: Option<Vec<TokenAndRange>>,
+    tokens: Option<Vec<TokenAndRange<'a>>>,
 }
 
-impl Context {
-    pub fn scan(&mut self) -> Result<Option<Token>, ParseError> {
+impl<'a> Context<'a> {
+    pub fn scan(&mut self) -> Result<Option<Token<'a>>, ParseError> {
         let previous_end = self.last_token_end;
         let token = self.scan_handling_comments()?;
         self.last_token_end = self.scanner.token_end();
@@ -76,7 +77,7 @@ impl Context {
         Ok(token)
     }
 
-    pub fn token(&self) -> Option<Token> {
+    pub fn token(&self) -> Option<Token<'a>> {
         self.scanner.token()
     }
 
@@ -117,7 +118,7 @@ impl Context {
         self.scanner.create_error_for_range(range, message)
     }
 
-    fn scan_handling_comments(&mut self) -> Result<Option<Token>, ParseError> {
+    fn scan_handling_comments(&mut self) -> Result<Option<Token<'a>>, ParseError> {
         loop {
             let token = self.scanner.scan()?;
             match token {
@@ -138,7 +139,7 @@ impl Context {
         }
     }
 
-    fn handle_comment(&mut self, comment: Comment) {
+    fn handle_comment(&mut self, comment: Comment<'a>) {
         if self.comments.is_some() {
             if let Some(comments) = self.current_comments.as_mut() {
                 comments.push(comment);
@@ -162,14 +163,25 @@ impl Context {
 /// }).expect("Should parse.");
 /// // ...inspect parse_result for value, tokens, and comments here...
 /// ```
-pub fn parse_to_ast(text: &str, options: &ParseOptions) -> Result<ParseResult, ParseError> {
+pub fn parse_to_ast<'a>(
+    text: &'a str,
+    options: &ParseOptions,
+) -> Result<ParseResult<'a>, ParseError> {
     let mut context = Context {
         scanner: Scanner::new(text),
-        comments: if options.comments { Some(HashMap::new()) } else { None },
+        comments: if options.comments {
+            Some(HashMap::new())
+        } else {
+            None
+        },
         current_comments: None,
         last_token_end: 0,
         range_stack: Vec::new(),
-        tokens: if options.tokens { Some(Vec::new()) } else { None },
+        tokens: if options.tokens {
+            Some(Vec::new())
+        } else {
+            None
+        },
     };
     context.scan()?;
     let value = parse_value(&mut context)?;
@@ -187,17 +199,25 @@ pub fn parse_to_ast(text: &str, options: &ParseOptions) -> Result<ParseResult, P
     })
 }
 
-fn parse_value(context: &mut Context) -> Result<Option<Value>, ParseError> {
+fn parse_value<'a>(context: &mut Context<'a>) -> Result<Option<Value<'a>>, ParseError> {
     match context.token() {
         None => Ok(None),
         Some(token) => match token {
             Token::OpenBrace => return Ok(Some(Value::Object(parse_object(context)?))),
             Token::OpenBracket => return Ok(Some(Value::Array(parse_array(context)?))),
-            Token::String(value) => return Ok(Some(Value::StringLit(create_string_lit(context, value)))),
-            Token::Boolean(value) => return Ok(Some(Value::BooleanLit(create_boolean_lit(context, value)))),
-            Token::Number(value) => return Ok(Some(Value::NumberLit(create_number_lit(context, value)))),
+            Token::String(value) => {
+                return Ok(Some(Value::StringLit(create_string_lit(context, value))))
+            }
+            Token::Boolean(value) => {
+                return Ok(Some(Value::BooleanLit(create_boolean_lit(context, value))))
+            }
+            Token::Number(value) => {
+                return Ok(Some(Value::NumberLit(create_number_lit(context, value))))
+            }
             Token::Null => return Ok(Some(Value::NullKeyword(create_null_keyword(context)))),
-            Token::CloseBracket => return Err(context.create_parse_error("Unexpected close bracket")),
+            Token::CloseBracket => {
+                return Err(context.create_parse_error("Unexpected close bracket"))
+            }
             Token::CloseBrace => return Err(context.create_parse_error("Unexpected close brace")),
             Token::Comma => return Err(context.create_parse_error("Unexpected comma")),
             Token::Colon => return Err(context.create_parse_error("Unexpected colon")),
@@ -208,7 +228,7 @@ fn parse_value(context: &mut Context) -> Result<Option<Value>, ParseError> {
     }
 }
 
-fn parse_object(context: &mut Context) -> Result<Object, ParseError> {
+fn parse_object<'a>(context: &mut Context<'a>) -> Result<Object<'a>, ParseError> {
     debug_assert!(context.token() == Some(Token::OpenBrace));
     let mut properties = Vec::new();
 
@@ -219,10 +239,10 @@ fn parse_object(context: &mut Context) -> Result<Object, ParseError> {
         match context.token() {
             Some(Token::CloseBrace) => break,
             Some(Token::String(prop_name)) => {
-                properties.push(parse_object_property(context, prop_name, true)?);
+                properties.push(parse_object_property(context, PropName::String(prop_name))?);
             }
             Some(Token::Word(prop_name)) | Some(Token::Number(prop_name)) => {
-                properties.push(parse_object_property(context, prop_name, false)?);
+                properties.push(parse_object_property(context, PropName::Word(prop_name))?);
             }
             None => return Err(context.create_parse_error_for_current_range("Unterminated object")),
             _ => return Err(context.create_parse_error("Unexpected token in object")),
@@ -243,22 +263,31 @@ fn parse_object(context: &mut Context) -> Result<Object, ParseError> {
     })
 }
 
-fn parse_object_property(
-    context: &mut Context,
-    prop_name: ImmutableString,
-    is_string: bool,
-) -> Result<ObjectProp, ParseError> {
+enum PropName<'a> {
+    String(Cow<'a, str>),
+    Word(&'a str),
+}
+
+fn parse_object_property<'a>(
+    context: &mut Context<'a>,
+    prop_name: PropName<'a>,
+) -> Result<ObjectProp<'a>, ParseError> {
     context.start_range();
 
-    let name = if is_string {
-        ObjectPropName::String(create_string_lit(context, prop_name))
-    } else {
-        ObjectPropName::Word(create_word(context, prop_name))
+    let name = match prop_name {
+        PropName::String(prop_name) => {
+            ObjectPropName::String(create_string_lit(context, prop_name))
+        }
+        PropName::Word(prop_name) => ObjectPropName::Word(create_word(context, prop_name)),
     };
 
     match context.scan()? {
         Some(Token::Colon) => {}
-        _ => return Err(context.create_parse_error("Expected a colon after the string or word in an object property")),
+        _ => {
+            return Err(context.create_parse_error(
+                "Expected a colon after the string or word in an object property",
+            ))
+        }
     }
 
     context.scan()?;
@@ -274,7 +303,7 @@ fn parse_object_property(
     }
 }
 
-fn parse_array(context: &mut Context) -> Result<Array, ParseError> {
+fn parse_array<'a>(context: &mut Context<'a>) -> Result<Array<'a>, ParseError> {
     debug_assert!(context.token() == Some(Token::OpenBracket));
     let mut elements = Vec::new();
 
@@ -287,7 +316,9 @@ fn parse_array(context: &mut Context) -> Result<Array, ParseError> {
             None => return Err(context.create_parse_error_for_current_range("Unterminated array")),
             _ => match parse_value(context)? {
                 Some(value) => elements.push(value),
-                None => return Err(context.create_parse_error_for_current_range("Unterminated array")),
+                None => {
+                    return Err(context.create_parse_error_for_current_range("Unterminated array"))
+                }
             },
         }
 
@@ -308,35 +339,35 @@ fn parse_array(context: &mut Context) -> Result<Array, ParseError> {
 
 // factory functions
 
-fn create_string_lit(context: &Context, value: ImmutableString) -> StringLit {
+fn create_string_lit<'a>(context: &Context<'a>, value: Cow<'a, str>) -> StringLit<'a> {
     StringLit {
         range: context.create_range_from_last_token(),
         value,
     }
 }
 
-fn create_word(context: &Context, value: ImmutableString) -> WordLit {
+fn create_word<'a>(context: &Context<'a>, value: &'a str) -> WordLit<'a> {
     WordLit {
         range: context.create_range_from_last_token(),
         value,
     }
 }
 
-fn create_boolean_lit(context: &Context, value: bool) -> BooleanLit {
+fn create_boolean_lit<'a>(context: &Context<'a>, value: bool) -> BooleanLit {
     BooleanLit {
         range: context.create_range_from_last_token(),
         value,
     }
 }
 
-fn create_number_lit(context: &Context, value: ImmutableString) -> NumberLit {
+fn create_number_lit<'a>(context: &Context<'a>, value: &'a str) -> NumberLit<'a> {
     NumberLit {
         range: context.create_range_from_last_token(),
         value,
     }
 }
 
-fn create_null_keyword(context: &Context) -> NullKeyword {
+fn create_null_keyword<'a>(context: &Context<'a>) -> NullKeyword {
     NullKeyword {
         range: context.create_range_from_last_token(),
     }
